function [VX, VY, NP, VL, VH, VSTEP] = read_hpp_vel(irange,flag,z)
%% [VX, VY, NP, LX, LY, VSTEP] = READ_HPP_VEL(IRANGE,FLAG,Z)
%%   Reads the binary files HPP_VEL with indexes in IRANGE.
%%   Returns two 3D matrices VX and VY of size NPxVHxVL that contain the data from the last file.
%%   If FLAG>=1, displays the velocity field of the plane Z (or the middle plane, by default).
%%   If FLAG=2, saves the figures.

%close all
%clear all

flag_fig=0;
flag_sav_fig=0;

if exist('flag','var')
  %% figure display flag
  flag_fig=(flag>=1);

  %% figure saving flag
  flag_sav_fig=(flag==2);
end

%% range of indexes
%irange=1:1;

for i=irange
  %% filename
  %%name = sprintf('HPP%05d_%d_%d_%d.vel', i, NP, VH, VL);
  name = sprintf('HPP_VEL%05d.bin', i);

  %% open HPP velocity file
  fprintf('reading %s\n',name);
  fid=fopen(name,'r');

  %% read header
  NP=fread(fid,1,'int');
  VL=fread(fid,1,'int');
  VH=fread(fid,1,'int');
  VSTEP=fread(fid,1,'int');
  fprintf('%d vertical planes of size %dx%d\n', NP, VL, VH);

  %% read velocity data
  a=fread(fid,2*NP*VH*VL,'int');
  fclose(fid);

  %% reshape data
  data=reshape(a,VL,VH,NP,2);
  VX=squeeze(data(:,:,:,1));
  VY=squeeze(data(:,:,:,2));

  if flag_fig
    %% middle plane
    if ~exist('z','var')
      z=int32(NP/2);
    elseif (z<1) | (z>NP)
      fprintf('ERROR: incorrect value z=%d\n',z);
      return;
    end

    %% velocity vectors
    velx=squeeze(VX(:,:,z))';
    vely=squeeze(VY(:,:,z))';
    vel=sqrt(velx.*velx+vely.*vely);
    %fprintf('mean velocity of plane %d: %f\n', z, mean(mean(vel)));

    %% plane grid with steps of 5 l0
    x=VSTEP*(1:VL);
    y=VSTEP*(1:VH);

    %% display absolute velocity
    figure
    imagesc(x,y,vel)
    xlabel('x [l_0]')
    ylabel('y [l_0]')
    daspect([1 1 1])
    colorbar('SouthOutside')
    %caxis([0 1200]);
    if flag_sav_fig
      imgname=sprintf('HPP_VEL%05d.png',i);
      fprintf('saving %s\n',imgname);
      print('-dpng',imgname);
    end

    %% display vector field
    figure
    quiver(x,y,velx,vely)
    xlabel('x [l_0]')
    ylabel('y [l_0]')
    daspect([1 1 1])
    set(gca, 'YDir', 'reverse')
    axis([0 VSTEP*VL 0 VSTEP*VH])
    if flag_sav_fig
      imgname=sprintf('HPP_VEL%05d_vec.png',i);
      fprintf('saving %s\n',imgname);
      print('-dpng',imgname);
    end

    %% display stream lines
    figure
    [sx sy] = meshgrid(1:5:VL, 1:2:VH);
    streamline(x,y,velx,vely,VSTEP*sx,VSTEP*sy) 
    xlabel('x [l_0]')
    ylabel('y [l_0]')
    daspect([1 1 1])
    set(gca, 'YDir', 'reverse')
    axis([0 VSTEP*VL 0 VSTEP*VH])
    if flag_sav_fig
      imgname=sprintf('HPP_VEL%05d_stream.png',i);
      fprintf('saving %s\n',imgname);
      print('-dpng',imgname);
    end
  end
end

